@@ -24,6 +24,7 @@ mod errors;
 mod execute;
 mod module;
 mod plugin;
+mod reporter;
 mod request;
 mod tag;
 mod util;
@@ -83,6 +84,22 @@ const SKYWALKING_AGENT_HEARTBEAT_PERIOD: &str = "skywalking_agent.heartbeat_peri
 const SKYWALKING_AGENT_PROPERTIES_REPORT_PERIOD_FACTOR: &str =
     "skywalking_agent.properties_report_period_factor";
 
+/// Whether to use zend observer instead of zend_execute_ex to hook the
+/// functions. This feature is only available for PHP8+, and can work with
+/// PHP8's jit.
+const SKYWALKING_AGENT_ENABLE_ZEND_OBSERVER: &str = "skywalking_agent.enable_zend_observer";
+
+/// Reporter type, optional values are `grpc` and `kafka`, default is `grpc`.
+const SKYWALKING_AGENT_REPORTER_TYPE: &str = "skywalking_agent.reporter_type";
+
+/// A list of host/port pairs to use for establishing the initial connection to
+/// the Kafka cluster. Only available when the reporter type is `kafka`.
+const SKYWALKING_AGENT_KAFKA_BOOTSTRAP_SERVERS: &str = "skywalking_agent.kafka_bootstrap_servers";
+
+/// Configure Kafka Producer configuration in JSON format.
+/// Only available when the reporter type is `kafka`.
+const SKYWALKING_AGENT_KAFKA_PRODUCER_CONFIG: &str = "skywalking_agent.kafka_producer_config";
+
 #[php_get_module]
 pub fn get_module() -> Module {
     let mut module = Module::new(
@@ -147,6 +164,22 @@ pub fn get_module() -> Module {
         10i64,
         Policy::System,
     );
+    module.add_ini(SKYWALKING_AGENT_ENABLE_ZEND_OBSERVER, false, Policy::System);
+    module.add_ini(
+        SKYWALKING_AGENT_REPORTER_TYPE,
+        "grpc".to_string(),
+        Policy::System,
+    );
+    module.add_ini(
+        SKYWALKING_AGENT_KAFKA_BOOTSTRAP_SERVERS,
+        "".to_string(),
+        Policy::System,
+    );
+    module.add_ini(
+        SKYWALKING_AGENT_KAFKA_PRODUCER_CONFIG,
+        "{}".to_string(),
+        Policy::System,
+    );
 
     // Hooks.
     module.on_module_init(module::init);
