@@ -16,26 +16,22 @@
 use crate::{
     channel::{self, TxReporter},
     module::{
-        AUTHENTICATION, ENABLE_TLS, HEARTBEAT_PERIOD, PROPERTIES_REPORT_PERIOD_FACTOR,
-        SERVICE_INSTANCE, SERVICE_NAME, SOCKET_FILE_PATH, SSL_CERT_CHAIN_PATH, SSL_KEY_PATH,
-        SSL_TRUSTED_CA_PATH,
+        HEARTBEAT_PERIOD, PROPERTIES_REPORT_PERIOD_FACTOR, SERVICE_INSTANCE, SERVICE_NAME,
+        SOCKET_FILE_PATH, WORKER_THREADS,
     },
+    reporter::run_reporter,
     util::change_permission,
-    SKYWALKING_AGENT_SERVER_ADDR, SKYWALKING_AGENT_WORKER_THREADS,
 };
-use anyhow::anyhow;
+
 use once_cell::sync::Lazy;
-use phper::ini::ini_get;
+
 use skywalking::{
     management::{instance::Properties, manager::Manager},
-    reporter::{
-        grpc::{CollectItemConsume, GrpcReporter},
-        CollectItem,
-    },
+    reporter::{CollectItem, CollectItemConsume},
 };
 use std::{
-    cmp::Ordering, error::Error, ffi::CStr, fs, io, marker::PhantomData, num::NonZeroUsize,
-    process::exit, thread::available_parallelism, time::Duration,
+    cmp::Ordering, error::Error, fs, io, marker::PhantomData, num::NonZeroUsize, process::exit,
+    thread::available_parallelism, time::Duration,
 };
 use tokio::{
     net::UnixListener,
@@ -43,19 +39,11 @@ use tokio::{
     select,
     signal::unix::{signal, SignalKind},
     sync::mpsc::{self, error::TrySendError},
-    time::sleep,
-};
-use tonic::{
-    async_trait,
-    transport::{Certificate, Channel, ClientTlsConfig, Endpoint, Identity},
 };
+use tonic::async_trait;
 use tracing::{debug, error, info, warn};
 
 pub fn init_worker() {
-    let server_addr = ini_get::<Option<&CStr>>(SKYWALKING_AGENT_SERVER_ADDR)
-        .and_then(|s| s.to_str().ok())
-        .unwrap_or_default()
-        .to_owned();
     let worker_threads = worker_threads();
 
     unsafe {
@@ -75,7 +63,7 @@ pub fn init_worker() {
 
                 // Run the worker in subprocess.
                 let rt = new_tokio_runtime(worker_threads);
-                match rt.block_on(start_worker(server_addr)) {
+                match rt.block_on(start_worker()) {
                     Ok(_) => {
                         exit(0);
                     }
@@ -91,7 +79,7 @@ pub fn init_worker() {
 }
 
 fn worker_threads() -> usize {
-    let worker_threads = ini_get::<i64>(SKYWALKING_AGENT_WORKER_THREADS);
+    let worker_threads = *WORKER_THREADS;
     if worker_threads <= 0 {
         available_parallelism().map(NonZeroUsize::get).unwrap_or(1)
     } else {
@@ -108,7 +96,7 @@ fn new_tokio_runtime(worker_threads: usize) -> Runtime {
         .unwrap()
 }
 
-async fn start_worker(server_addr: String) -> anyhow::Result<()> {
+async fn start_worker() -> anyhow::Result<()> {
     debug!("Starting worker...");
 
     // Ensure to cleanup resources when worker exits.
@@ -170,30 +158,10 @@ async fn start_worker(server_addr: String) -> anyhow::Result<()> {
             }
         });
 
-        let endpoint = create_endpoint(&server_addr).await?;
-        let channel = connect(endpoint).await;
-
         report_properties_and_keep_alive(TxReporter(tx_));
 
-        let mut reporter = GrpcReporter::new_with_pc(channel, (), Consumer(rx));
-
-        if !AUTHENTICATION.is_empty() {
-            reporter = reporter.with_authentication(&*AUTHENTICATION);
-        }
-
-        info!("Worker is ready...");
-
-        let handle = reporter
-            .reporting()
-            .await
-            .with_status_handle(|message, status| {
-                warn!(?status, "Collect failed: {}", message);
-            })
-            .spawn();
-
-        handle
-            .await
-            .map_err(|err| anyhow!("Tracer reporting failed: {:?}", err))?;
+        // Run reporter with blocking.
+        run_reporter((), Consumer(rx)).await?;
 
         Ok::<_, anyhow::Error>(())
     };
@@ -212,60 +180,6 @@ async fn start_worker(server_addr: String) -> anyhow::Result<()> {
     Ok(())
 }
 
-async fn create_endpoint(server_addr: &str) -> anyhow::Result<Endpoint> {
-    let scheme = if *ENABLE_TLS { "https" } else { "http" };
-
-    let url = format!("{}://{}", scheme, server_addr);
-    debug!(url, "Create Endpoint");
-    let mut endpoint = Endpoint::from_shared(url)?;
-
-    if *ENABLE_TLS {
-        let domain_name = server_addr.split(':').next().unwrap_or_default();
-        debug!(domain_name, "Configure TLS domain");
-        let mut tls = ClientTlsConfig::new().domain_name(domain_name);
-
-        let ssl_trusted_ca_path = SSL_TRUSTED_CA_PATH.as_str();
-        if !ssl_trusted_ca_path.is_empty() {
-            debug!(ssl_trusted_ca_path, "Configure TLS CA");
-            let ca_cert = tokio::fs::read(&*SSL_TRUSTED_CA_PATH).await?;
-            let ca_cert = Certificate::from_pem(ca_cert);
-            tls = tls.ca_certificate(ca_cert);
-        }
-
-        let ssl_key_path = SSL_KEY_PATH.as_str();
-        let ssl_cert_chain_path = SSL_CERT_CHAIN_PATH.as_str();
-        if !ssl_key_path.is_empty() && !ssl_cert_chain_path.is_empty() {
-            debug!(ssl_trusted_ca_path, "Configure mTLS");
-            let client_cert = tokio::fs::read(&*SSL_CERT_CHAIN_PATH).await?;
-            let client_key = tokio::fs::read(&*SSL_KEY_PATH).await?;
-            let client_identity = Identity::from_pem(client_cert, client_key);
-            tls = tls.identity(client_identity);
-        }
-
-        endpoint = endpoint.tls_config(tls)?;
-    }
-
-    Ok(endpoint)
-}
-
-#[tracing::instrument(skip_all)]
-async fn connect(endpoint: Endpoint) -> Channel {
-    let channel = loop {
-        match endpoint.connect().await {
-            Ok(channel) => break channel,
-            Err(err) => {
-                warn!(?err, "Connect to skywalking server failed, retry after 10s");
-                sleep(Duration::from_secs(10)).await;
-            }
-        }
-    };
-
-    let uri = &*endpoint.uri().to_string();
-    info!(uri, "Skywalking server connected");
-
-    channel
-}
-
 struct Consumer(mpsc::Receiver<CollectItem>);
 
 #[async_trait]
