@@ -56,6 +56,7 @@ async fn run_e2e() {
     request_fpm_memcached().await;
     request_fpm_redis().await;
     request_fpm_rabbitmq().await;
+    request_fpm_mongodb().await;
     request_swoole_curl().await;
     request_swoole_2_curl().await;
     request_swoole_2_pdo().await;
@@ -63,6 +64,7 @@ async fn run_e2e() {
     request_swoole_2_memcached().await;
     request_swoole_2_redis().await;
     request_swoole_2_predis().await;
+    request_swoole_2_mongodb().await;
     sleep(Duration::from_secs(3)).await;
     request_collector_validate().await;
 }
@@ -140,6 +142,14 @@ async fn request_fpm_rabbitmq() {
     .await;
 }
 
+async fn request_fpm_mongodb() {
+    request_common(
+        HTTP_CLIENT.get(format!("http://{}/mongodb.php", PROXY_SERVER_1_ADDRESS)),
+        "ok",
+    )
+    .await;
+}
+
 async fn request_swoole_curl() {
     request_common(
         HTTP_CLIENT.get(format!("http://{}/curl", SWOOLE_SERVER_1_ADDRESS)),
@@ -196,6 +206,14 @@ async fn request_swoole_2_predis() {
     .await;
 }
 
+async fn request_swoole_2_mongodb() {
+    request_common(
+        HTTP_CLIENT.get(format!("http://{}/mongodb", SWOOLE_SERVER_2_ADDRESS)),
+        "ok",
+    )
+    .await;
+}
+
 async fn request_collector_validate() {
     request_common(
         HTTP_CLIENT
@@ -215,6 +233,6 @@ async fn request_common(request_builder: RequestBuilder, actual_content: impl In
     let response = request_builder.send().await.unwrap();
     let status = response.status();
     let content = response.text().await.unwrap();
-    info!(content, "response content");
+    info!("response content: {}", content);
     assert_eq!((status, content), (StatusCode::OK, actual_content.into()));
 }
